﻿using System;
using System.Text;
using System.Windows.Forms;
using System.Configuration;
using System.IO;
using DJTRADEOBJLibWLS;
using System.Reflection;

namespace TradeObjTest
{
    /// <summary>
    /// 附錄D：帳號回傳格式，欄位順序
    /// </summary>
    public enum AccountResponse
    {
        ID = 0,
        Name = 1,
        UID = 2,
        Type = 3
    }

    /// <summary>
    /// 適用所有"欄位=值"，Split後的順序
    /// </summary>
    public enum KeyValue
    {
        Key = 0,  // 欄位
        Value = 1 // 值
    }

    public partial class Form1 : Form
    {
        TradeAppClass m_tradeApp;
        int m_nTradeDate = 0;
        int m_nSTK_Conn = 0;
        int m_nFUT_Conn = 0;
        int m_nHK_Conn = 0;
        bool m_bLoginOK = false;
        string jasondata = "";

        public Form1()
        {
            InitializeComponent();
        }

        void Init()
        {
            AppSettingsReader ASReader = new AppSettingsReader();

            txt_UID.Text = ASReader.GetValue("UserID", typeof(string)).ToString();
            txt_Pwd.Text = ASReader.GetValue("UserPWD", typeof(string)).ToString();

            txt_CA_Path.Text = ASReader.GetValue("CAPath", typeof(string)).ToString();
            txt_CA_Pwd.Text = ASReader.GetValue("CAPWD", typeof(string)).ToString();

            string strTradeDAS = "";
            strTradeDAS = ASReader.GetValue("TradeDas", typeof(string)).ToString();
            string strLoginServer = "登入主機位置:[" + strTradeDAS + "]";
            this.Text += "  " + strLoginServer ;

            InitTradeApp();

            int nErrCode = 0;
            string sErrMsg = "";
            int nRet = m_tradeApp.Init(strTradeDAS, out nErrCode, out sErrMsg);

            // LotSize example
            //string strLotSizeData = "0050=300|0028=200";
            //m_tradeApp.SetLotSizeData(strLotSizeData);

            DateTime dTDate = DateTime.Now;
            m_nTradeDate = dTDate.Year * 10000 + dTDate.Month * 100 + dTDate.Day;
            txt_STK_TradeDate.Text = m_nTradeDate.ToString();
            txt_TFX_TradeDate.Text = m_nTradeDate.ToString();

            ShowConnectStatus();
        }

        void InitTradeApp()
        {
            m_tradeApp = new TradeAppClass();
            m_tradeApp.OnDataResponse += new _ITradeAppEvents_OnDataResponseEventHandler(m_tradeApp_OnDataResponse);
        }

        void m_tradeApp_OnDataResponse(int nEventID, string sResponseData)
        {
            switch (nEventID)
            {
                case 1: // 主動回報 連線狀態
                    this.BeginInvoke((MethodInvoker)delegate()
                    {
                        SetConnectStatus(sResponseData);
                    });
                    break;

                case 100: // 主動回報
                    this.BeginInvoke((MethodInvoker)delegate()
                    {
                        txt_MSN.Text += sResponseData + "\r\n";
                    });
                    break;

                case 101: // 下單 回傳(非同步)
                case 102: // 交易查詢 回傳(非同步)
                    this.BeginInvoke((MethodInvoker)delegate ()
                    {
                        txt_Result.Text = sResponseData.Replace(">", ">\r\n");
                    });
                    break;

                case 103: // 帳務查詢 回傳(非同步)
                    this.BeginInvoke((MethodInvoker)delegate()
                    {
                        txt_Result.Text = sResponseData.Replace(">", ">\r\n");
                    });
                    break;
            }
        }

        void SetConnectStatus(string sData)
        {
            string[] aryConn = sData.Split(',');

	        if (aryConn.Length >= 2)
	        {
                int nConn = Int32.Parse(aryConn[1]);
                switch (Int32.Parse(aryConn[0]))
                {
                    case 1: // 證券
                        m_nSTK_Conn = nConn;
                        break;
                    case 2: // 期權
                        m_nFUT_Conn = nConn;
                        break;
                    case 3: // 複委託
                        m_nHK_Conn = nConn;
                        break;
                }

                ShowConnectStatus();
	        }
        }

        void ShowConnectStatus()
        {
            string sConnStatus = string.Format("連線狀態:\r\n證券: {0}\r\n期權: {1}",
                                    (m_nSTK_Conn == 0) ? "" : "已連線",
                                    (m_nFUT_Conn == 0) ? "" : "已連線");

            txt_ConnStatus.Text = sConnStatus;
        }

        void Fini()
        {
            try
            {
                m_tradeApp.Fini();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                MessageBox.Show(ex.Message);

                Application.Exit();
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                Init();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                MessageBox.Show(ex.Message);

                Application.Exit();
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (MessageBox.Show("Are you sure want to quit?", "關閉程式", MessageBoxButtons.YesNo)
                == DialogResult.No)
            {
                e.Cancel = true;
            }
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            Fini();
        }

        private void SetAccountList()
        {
            cb_Account.Items.Clear();
            cb_STK_Account.Items.Clear();
            cb_TFX_Account.Items.Clear();
            int nCount = m_tradeApp.GetAccountCount(); // 取得所有帳號數量

            for (int i = 0; i < nCount; i++)
            {
                string AccountData = m_tradeApp.GetAccount(i);
                AccountData = AccountData.Trim("<>".ToCharArray()); // 移除 "<" 和 ">"
                cb_Account.Items.Add(AccountData);

                string[] aryField = AccountData.Split('|'); // 取得所有的"欄位=值"
                char separator = '='; // 取得"欄位"和"值"
                if (aryField.Length >= Enum.GetNames(typeof(AccountResponse)).Length)
                {
                    string[] aryAccID = aryField[(int)AccountResponse.ID].Split(separator);
                    string[] aryAccName = aryField[(int)AccountResponse.Name].Split(separator);
                    string[] aryUID = aryField[(int)AccountResponse.UID].Split(separator);
                    string[] aryType = aryField[(int)AccountResponse.Type].Split(separator);

                    int nAccType = Int32.Parse(aryType[(int)KeyValue.Value]);
                    string strAccountFullName = aryAccName[(int)KeyValue.Value] + "|" + aryAccID[(int)KeyValue.Value];
                    switch (nAccType)
                    {
                        case 1: // 證券
                            cb_STK_Account.Items.Add(strAccountFullName);
                            break;
                        case 2: // 期權
                            cb_TFX_Account.Items.Add(strAccountFullName);
                            break;
                    }
                    cb_AllAccount.Items.Add(strAccountFullName);
                }
            }

            if (cb_Account.Items.Count > 0)
                cb_Account.SelectedIndex = 0;
            if (cb_STK_Account.Items.Count > 0)
                cb_STK_Account.SelectedIndex = 0;
            if (cb_TFX_Account.Items.Count > 0)
                cb_TFX_Account.SelectedIndex = 0;
            if (cb_AllAccount.Items.Count > 0)
                cb_AllAccount.SelectedIndex = 0;
        }

        /// <summary>
        /// 根據有價證券種類取得被選取的帳號
        /// </summary>
        /// <param name="Type">"證券"、"期貨/複委託"和"全部"</param>
        /// <returns></returns>
        private string GetAccountBySecuritiesType(string Type)
        {
            string strSelectedAccount = string.Empty;
            string[] aryAcc = null;
            string TypeInUpperCase = Type.ToUpper();

            if (TypeInUpperCase == Securities.STOCK)
            {
                aryAcc = cb_STK_Account.Text.Split('|');
            }
            else if (TypeInUpperCase == Securities.TFX)
            {
                aryAcc = cb_TFX_Account.Text.Split('|');
            }
            else if (TypeInUpperCase == Securities.ALL)
            {
                aryAcc = cb_AllAccount.Text.Split('|');
            }

            strSelectedAccount = (aryAcc.Length > 1) ? aryAcc[(int)KeyValue.Value] : string.Empty;

            return strSelectedAccount;
        }

        #region 設定相關功能(登入、登出、連線、斷線)

        /// <summary>
        /// 登入
        /// </summary>
        private void btn_Login_Click(object sender, EventArgs e)
        {
            int nErrCode = 0;
            string sErrMsg = "";
            try
            {
                int iStatusCode = m_tradeApp.Login(txt_UID.Text.Trim(), txt_Pwd.Text.Trim(), txt_CA_Path.Text.Trim(), txt_CA_Pwd.Text.Trim(), out nErrCode, out sErrMsg);
                if (iStatusCode != 0)
                {
                    btn_Login.Enabled = false;
                    SetAccountList();

                    if (cb_STK_Account.Items.Count > 0)
                    {
                        groupStock.Enabled = true;
                    }

                    if (cb_TFX_Account.Items.Count > 0)
                    {
                        groupFutures.Enabled = true;
                    }

                    //if (ALLAccount.Items.Count > 0)
                    //{
                    //    groupMIXQuery.Enabled = true;
                    //}

                    groupUnitTest.Enabled = true;

                    MessageBox.Show("登入成功");
                    m_bLoginOK = true;
                }
                else
                {
                    MessageBox.Show(string.Format("登入失敗:{0}", sErrMsg));
                }
            }
            catch(Exception)
            {
                MessageBox.Show("登入失敗");
            }
        }

        /// <summary>
        /// 登出
        /// </summary>
        private void btn_Logout_Click(object sender, EventArgs e)
        {
            m_tradeApp.Logout(txt_UID.Text.Trim());
            btn_Login.Enabled = true;

            cb_Account.Items.Clear();
            cb_Account.Text = "";

            cb_STK_Account.Items.Clear();
            cb_STK_Account.Text = "";

            cb_TFX_Account.Items.Clear();
            cb_TFX_Account.Text = "";

            cb_AllAccount.Items.Clear();
            cb_AllAccount.Text = "";

            groupStock.Enabled = false;
            groupFutures.Enabled = false;
            //groupMIXQuery.Enabled = false;
            groupUnitTest.Enabled = false;

            m_bLoginOK = false;
        }

        /// <summary>
        /// 連線
        /// </summary>
        private void btn_Connect_Click(object sender, EventArgs e)
        {
            if (m_bLoginOK == false)
            {
                MessageBox.Show("請先登入");
                return;
            }

            if (m_tradeApp.Connect() != 0)
            {
                btn_Connect.Enabled = false;
            }
        }

        /// <summary>
        /// 斷線
        /// </summary>
        private void btn_Disconnect_Click(object sender, EventArgs e)
        {
            m_tradeApp.Disconnect();

            m_nSTK_Conn = 0;
            m_nFUT_Conn = 0;
            m_nHK_Conn = 0;

            btn_Connect.Enabled = true;
            ShowConnectStatus();
        }

        #endregion

        #region 簽章測試

        private void btn_CA_Path_Click(object sender, EventArgs e)
        {
            if (openFileDlg.ShowDialog() == DialogResult.OK)
            {
                txt_CA_Path.Text = openFileDlg.FileName;
            }
        }

        /// <summary>
        /// 簽章記憶
        /// </summary>
        private void btn_CAConfig_Click(object sender, EventArgs e)
        {
            if (txt_UID.Text.Trim().Length <= 0 || txt_CA_Path.Text.Trim().Length <= 0 || txt_CA_Pwd.Text.Trim().Length <= 0)
            {
                MessageBox.Show("請設定憑證檔案及密碼");
                return;
            }

            if (File.Exists(txt_CA_Path.Text.Trim()) == false)
            {
                MessageBox.Show("憑證檔案不存在");
                return;
            }

            if (m_tradeApp.SetCAPathAndPW(txt_UID.Text.Trim(), txt_CA_Path.Text.Trim(), txt_CA_Pwd.Text.Trim()) == 1)
            {
                MessageBox.Show("設定成功");
            }
            else
            {
                MessageBox.Show("設定失敗");
            }
        }

        /// <summary>
        /// 簽章測試
        /// </summary>
        private void btn_CATest_Click(object sender, EventArgs e)
        {
            string[] aryField = cb_Account.Text.Split('|');
            if (aryField.Length >= Enum.GetNames(typeof(AccountResponse)).Length)
            {
                string[] aryAccID = aryField[(int)AccountResponse.ID].Split('=');
                string error = m_tradeApp.CASignCheck(aryAccID[(int)KeyValue.Value]);

                if (string.IsNullOrEmpty(error))
                    MessageBox.Show("簽章成功");
                else
                    MessageBox.Show("簽章失敗 - " + error);
            }
            else
            {
                MessageBox.Show("簽章失敗 - 無可用交易帳號");
            }
        }

        /// <summary>
        /// 清除記錄
        /// </summary>
        private void btn_ClearPathAndPwd_Click(object sender, EventArgs e)
        {
            m_tradeApp.ClearPathAndPW();
            MessageBox.Show("清除成功");
        }

        #endregion

        #region 證券

        #region 委託

        /// <summary>
        /// 新單
        /// </summary>
        private void btn_STK_NewOrder_Click(object sender, EventArgs e)
        {
            m_nTradeDate = Check_STK_TradeDate();
            if (m_nTradeDate < 0)
            {
                MessageBox.Show("交易日期格式錯誤", "錯誤");
                return;
            }

            string szAccountID = GetAccountBySecuritiesType(Securities.STOCK); // 取得證券帳號
            int nTT = Get_STK_TT();
	        int nOT = Get_STK_OT();
            int nBS = Get_STK_BS(); // 1: 買進/ 2: 賣出
	        string szStockID = txt_StockID.Text.Trim();

            if (!IsNumber(txt_STK_Qty.Text.Trim()))
            {
                MessageBox.Show("請輸入數量");
                return;
            }
            int nQty = Convert.ToInt32(txt_STK_Qty.Text.Trim());

            string szPrice = txt_STK_Price.Text.Trim();

            int nPT = Get_STK_PT(szPrice);
            if (nPT < 0)
            {
                MessageBox.Show("價格不正確!");
                return;
            }

            string szBroker = "";
	        int nPayType = 0;
            int nPreOrder = STK_CheckPreOrder.Checked ? 1 : 0;

            //委託條件 0 : ROD, 1: FOK, 2: IOC
            string sCond = string.Empty;
            sCond = Get_OrderCondition(cb_STK_Cond.Text.Trim());

            string strResult = m_tradeApp.Stock_NewOrder(szAccountID, m_nTradeDate, nTT, nOT, nBS, szStockID, nQty, nPT, szPrice, szBroker, nPayType, sCond, jasondata); // 預約單不用加一天

            string sParameters =
                string.Format("sAccountID = '{0}', lTradeDate = '{1}', nTT = '{2}', nOT = '{3}', nBS = '{4}', sStockID = '{5}', lQty = '{6}', nPT '{7}', sPrice = '{8}', sBroker = '{9}', nPayType = '{10}', sCond = '{11}'",
                szAccountID, m_nTradeDate, nTT, nOT, nBS, szStockID, nQty, nPT, szPrice, szBroker, nPayType, sCond);

            OutToMessage("證券新單：" + strResult + "\r\n" + "參數：" + sParameters);

            ShowOrderNo(strResult);
        }

        /// <summary>
        /// 改價
        /// </summary>
        private void btn_STK_ModifyOrderPrice_Click(object sender, EventArgs e)
        {
            m_nTradeDate = Check_STK_TradeDate();
            if (m_nTradeDate < 0)
            {
                MessageBox.Show("交易日期格式錯誤", "錯誤");
                return;
            }

            Modify_STK_Order(true);
        }

        /// <summary>
        /// 改量
        /// </summary>
        private void btn_STK_ModifyOrderQty_Click(object sender, EventArgs e)
        {
            m_nTradeDate = Check_STK_TradeDate();
            if (m_nTradeDate < 0)
            {
                MessageBox.Show("交易日期格式錯誤", "錯誤");
                return;
            }

            Modify_STK_Order(false);
        }

        /// <summary>
        /// 證券改單
        /// </summary>
        /// <param name="bChangePrice">true:改價；false:改量</param>
        private void Modify_STK_Order(bool bChangePrice)
        {
            string szAccountID = GetAccountBySecuritiesType(Securities.STOCK);
            int nTT = Get_STK_TT();
            int nOT = Get_STK_OT();
            string szOID = string.Empty;
            string szOrderNo = txt_STK_OrderNo.Text.Trim();
            string sStockID = txt_StockID.Text.Trim();
            int nBS = Get_STK_BS();    // 1: 買進/ 2: 賣出

            int nQty = 0;
            if (bChangePrice == false)
            {
                if (!IsNumber(txt_STK_Qty.Text.Trim()))
                {
                    MessageBox.Show("請輸入數量");
                    return;
                }
                nQty = Convert.ToInt32(txt_STK_Qty.Text.Trim());
            }

            int nQCurrent = 0;
            int nQMatch = 0;
            int nPreOrder = STK_CheckPreOrder.Checked ? 1 : 0;

            string szNewPrice = txt_STK_Price.Text.Trim();
            string szNewPT = Get_STK_PT(szNewPrice).ToString();

            //委託條件 0 : ROD, 1: FOK, 2: IOC
            string szNewCond = string.Empty;
            szNewCond = Get_OrderCondition(cb_STK_Cond.Text.Trim());

            string szCond = szNewCond;

            string strResult = m_tradeApp.Stock_ModifyOrder(szAccountID, m_nTradeDate, nTT, nOT, szOID, szOrderNo, sStockID,
                                                            nBS, nQty, nQCurrent, nQMatch, nPreOrder, szNewPrice, szNewPT, Convert.ToInt16(bChangePrice), szNewCond, szCond, jasondata); // 預約單不用加一天

            string sParameters =
                string.Format("sAccountID = '{0}', nTradeDate = '{1}', nTT = '{2}', nOT = '{3}', sOID = '{4}', sOrderNo = '{5}', sStockID = '{6}', nBS = '{7}', nQty = '{8}', nQcurrent = '{9}', nQmatch = '{10}', nPreOrder = '{11}', sNewPrice = '{12}', sNewPT = '{13}', bChangePrice = '{14}', sNewCond = '{15}', sCond = '{16}'",
                szAccountID, m_nTradeDate, nTT, nOT, szOID, szOrderNo, sStockID, nBS, nQty, nQCurrent, nQMatch, nPreOrder, szNewPrice, szNewPT, bChangePrice, szNewCond, szCond);

            OutToMessage("證券改單：" + strResult + "\r\n" + "參數：" + sParameters);
        }

        /// <summary>
        /// 刪單
        /// </summary>
        private void btn_STK_CancelOrder_Click(object sender, EventArgs e)
        {
            m_nTradeDate = Check_STK_TradeDate();
            if (m_nTradeDate < 0)
            {
                MessageBox.Show("交易日期格式錯誤", "錯誤");
                return;
            }

            string szAccountID = GetAccountBySecuritiesType("STK");
            int nTT = Get_STK_TT();
            int nOT = Get_STK_OT();
            string szOID = txt_STK_OID.Text.Trim();
            string szOrderNo = txt_STK_OrderNo.Text.Trim();
            string sStockID = txt_StockID.Text.Trim();
            int nBS = Get_STK_BS();    // 1: 買進/ 2: 賣出

            int nQty = 0;
            int nQCurrent = 0;
            int nQMatch = 0;

            int nPreOrder = STK_CheckPreOrder.Checked ? 1 : 0;
            string szPrice = txt_STK_Price.Text.Trim();

            //委託條件 0 : ROD, 1: FOK, 2: IOC
            string szCond = "";
            szCond = Get_OrderCondition(cb_STK_Cond.Text.Trim());

            string strResult = m_tradeApp.Stock_CancelOrder(szAccountID, m_nTradeDate, nTT, nOT,
                                                            szOID, szOrderNo, sStockID, nBS, nQty, nQCurrent, nQMatch, nPreOrder, szPrice, szCond, jasondata); // 預約單不用加一天

            string sParameters =
                string.Format("sAccountID = '{0}', nTradeDate = '{1}', nTT = '{2}', nOT = '{3}', sOID = '{4}', sOrderNo = '{5}', sStockID = '{6}', nBS = '{7}', nQty = '{8}', nQcurrent = '{9}', nQmatch = '{10}', nPreOrder = '{11}', szPrice = '{12}', sCond = '{13}'",
                szAccountID, m_nTradeDate, nTT, nOT, szOID, szOrderNo, sStockID, nBS, nQty, nQCurrent, nQMatch, nPreOrder, szPrice, szCond);

            OutToMessage("證券刪單：" + strResult + "\r\n" + "參數：" + sParameters);
        }

        #endregion

        #region 查詢

        /// <summary>
        /// 委託查詢
        /// </summary>
        private void btn_STK_QryOrder_Click(object sender, EventArgs e)
        {
            string szAccountID = GetAccountBySecuritiesType(Securities.STOCK);
            int nForceQuery = STK_CheckForceQuery.Checked ? 1 : 0;

            string strResult = m_tradeApp.Stock_QueryOrder(szAccountID, nForceQuery);

            OutToMessage("證券委託查詢：" + strResult);
        }

        /// <summary>
        /// 成交查詢
        /// </summary>
        private void btn_STK_QryMatch_Click(object sender, EventArgs e)
        {
            string szAccountID = GetAccountBySecuritiesType(Securities.STOCK);
            int nForceQuery = STK_CheckForceQuery.Checked ? 1 : 0;

            string strResult = m_tradeApp.Stock_QueryMatch(szAccountID, nForceQuery);

            OutToMessage("證券成交查詢：" + strResult);
        }

        /// <summary>
        /// 庫存查詢
        /// </summary>
        private void btn_STK_QryPosition_Click(object sender, EventArgs e)
        {
            m_nTradeDate = Check_STK_TradeDate();
            if (m_nTradeDate < 0)
            {
                MessageBox.Show("交易日期格式錯誤", "錯誤");
                return;
            }

            string szAccountID = GetAccountBySecuritiesType(Securities.STOCK);
            int nForceQuery = STK_CheckForceQuery.Checked ? 1 : 0;

            string strResult = m_tradeApp.Stock_QueryPosition(szAccountID, m_nTradeDate, nForceQuery);

            OutToMessage("證券庫存查詢：" + strResult);
        }

        #endregion

        #region 其他

        /// <summary>
        /// 檢核交易日期
        /// </summary>
        private int Check_STK_TradeDate()
        {
            int returnValue = -1;

            txt_STK_TradeDate.Text = txt_STK_TradeDate.Text.Trim();
            if (txt_STK_TradeDate.Text.Length == 8)
            {
                try
                {
                    DateTime.ParseExact(txt_STK_TradeDate.Text, "yyyyMMdd", null);
                    returnValue = Convert.ToInt32(txt_STK_TradeDate.Text);
                }
                catch (Exception)
                {
                    returnValue = -1;
                }
            }

            return returnValue;
        }

        /// <summary>
        /// 普通、零股、盤後和興櫃
        /// </summary>
        private int Get_STK_TT()
        {
            if (cb_STK_TT.SelectedIndex == 0 || cb_STK_TT.SelectedIndex == 1 || cb_STK_TT.SelectedIndex == 2)
            {
                return cb_STK_TT.SelectedIndex;
            }
            else if (cb_STK_TT.SelectedIndex == 3)
            {
                return 5; // 盤後
            }
            else if (cb_STK_TT.SelectedIndex == 4)
            {
                return 7; // 盤中零股
            }

            return 0;
        }

        /// <summary>
        /// 現股、融資、融券、先賣
        /// </summary>
        private int Get_STK_OT()
        {
            int nOT = 0;
            if (cb_STK_OT.SelectedIndex == 0 || cb_STK_OT.SelectedIndex == 1 || cb_STK_OT.SelectedIndex == 2)
            {
                nOT = cb_STK_OT.SelectedIndex;
            }
            else if (cb_STK_OT.SelectedIndex == 3)
            {
                nOT = 16;
            }
            else if (cb_STK_OT.SelectedIndex == 4)
            {
                nOT = 17;
            }
            else if (cb_STK_OT.SelectedIndex == 5)
            {
                nOT = 18;
            }
            return nOT;
        }

        /// <summary>
        /// 買進或賣出
        /// </summary>
        private int Get_STK_BS()
        {
            int nBS = 2;
            if (cb_STK_BS.SelectedIndex == 0 || cb_STK_BS.SelectedIndex == 1)
            {
                nBS = cb_STK_BS.SelectedIndex + 1;
            }
            return nBS;
        }

        /// <summary>
        /// 限價、漲停、跌停、平盤、市價
        /// </summary>
        /// <param name="szPrice">價格</param>
        private int Get_STK_PT(string szPrice)
        {
            int nPT = 0;
            switch (cb_STK_PT.Text)
            {
                case "限價":
                    if (szPrice.Length == 0 || szPrice == "0")
                    {
                        return -1;
                    }
                    nPT = 0;
                    break;
                case "漲停":
                    nPT = 1;
                    break;
                case "跌停":
                    nPT = 2;
                    break;
                case "平盤":
                    nPT = 3;
                    break;
                case "市價":
                    nPT = 4;
                    break;
            }

            return nPT;
        }

        /// <summary>
        /// 取得委託條件
        /// </summary>
        /// <param name="Condition">委託條件下拉選單</param>
        private string Get_OrderCondition(string Condition)
        {
            string szCond = string.Empty;

            if (Condition == OrderCondition.ROD)
                szCond = "0";
            else if (Condition == OrderCondition.FOK)
                szCond = "1";
            else if (Condition == OrderCondition.IOC)
                szCond = "2";

            return szCond;
        }

        /// <summary>
        /// 解析委託書號
        /// </summary>
        /// <param name="result">新單的一般訊息</param>
        private void ShowOrderNo(string result)
        {
            if (string.IsNullOrEmpty(result))
                return;

            int indexStart = result.IndexOf("F2=");
            int indexEnd = result.IndexOf("|F3");
            if (indexStart >= 0)
            {
                txt_STK_OrderNo.Text = result.Substring(indexStart + 3, indexEnd - indexStart - 3);
            }
        }

        #endregion

        #endregion

        #region 期權

        #region 委託

        /// <summary>
        /// 新單
        /// </summary>
        private void btn_TFX_NewOrder_Click(object sender, EventArgs e)
        {
            m_nTradeDate = Check_TFX_TradeDate();
            if (m_nTradeDate < 0)
            {
                MessageBox.Show("交易日期格式錯誤", "錯誤");
                return;
            }

            //string szAccountID = "15000-3314559";
            string szAccountID = GetAccountBySecuritiesType(Securities.TFX);

            int nTT = Get_TFX_Type(); // 期權商品類型 (0:期貨, 1:選擇權)
            string sTradeID1 = txt_TFX_TradeID.Text.Trim();
            int nBS1 = Get_TFX_BS();  // 買賣別 (1: 買進/ 2: 賣出/ 3:超買/ 4:超賣 (3/4 for 興櫃))
            
            string sPrice = txt_TFX_Price.Text.Trim();

            int nPT = 0; // PT (0:限價, 1:漲停, 2:跌停, 3:平盤, 4:市價, 5:長效單)

            switch (cb_TFX_PT.Text.Trim())
            {
                case "限價":
                    nPT = 0;

                    if (sPrice.Length == 0 || sPrice == "0")
                    {
                        MessageBox.Show("價格不正確!");
                        return;
                    }

                    break;
                case "漲停":
                    nPT = 1;
                    break;
                case "跌停":
                    nPT = 2;
                    break;
                case "平盤":
                    nPT = 3;
                    break;
                case "市價":
                    nPT = 4;
                    break;
            }

            if (!IsNumber(txt_TFX_Qty.Text.Trim()))
            {
                MessageBox.Show("請輸入數量");
                return;
            }
            int nQty = Convert.ToInt32(txt_TFX_Qty.Text.Trim());
            int nOffset = 0; // 倉別(Offset) 0:新倉, 1:平倉, 2:自動, 4:當沖

            // 0:ROD, 1:FOK, 2:IOC
            int nCond = 0;
            if (cb_TFX_Cond.Text == OrderCondition.ROD)
                nCond = 0;
            else if (cb_TFX_Cond.Text == OrderCondition.FOK)
                nCond = 1;
            else if (cb_TFX_Cond.Text == OrderCondition.IOC)
                nCond = 2;

            string strTradeID2 = "";
            int nBS2 = 0;
            int nPreOrder = TFX_CheckPreOrder.Checked ? 1 : 0; ;  // 1: 預約單/ 0:盤中單

            string strResult = m_tradeApp.FutOpt_NewOrder(szAccountID, m_nTradeDate, nTT, sTradeID1, nBS1,
                                nPT, sPrice, nQty, nOffset, nCond, strTradeID2, nBS2, nPreOrder, jasondata);

            OutToMessage("期貨新單：" + strResult);
        }

        /// <summary>
        /// 改單
        /// </summary>
        private void btn_TFX_ModifyOrder_Click(object sender, EventArgs e)
        {
            m_nTradeDate = Check_TFX_TradeDate();
            if (m_nTradeDate < 0)
            {
                MessageBox.Show("交易日期格式錯誤", "錯誤");
                return;
            }

            //string szAccountID = "15000-3314559";
            string szAccountID = GetAccountBySecuritiesType(Securities.TFX);
            int nType = 0;				// 0:改量, 1:刪單, 2:改價
            string szOID = txt_TFX_OID.Text.Trim();
            string szOrderNo = txt_TFX_OrderNo.Text.Trim();

            if (!IsNumber(txt_TFX_Qty.Text.Trim()))
            {
                MessageBox.Show("請輸入數量");
                return;
            }
            int nQty = Convert.ToInt32(txt_TFX_Qty.Text.Trim());

            int nTT = Get_TFX_Type();    // 期權商品類型 (0:期貨, 1:選擇權)

            if (!IsNumber(txt_TFX_QCurrent.Text.Trim()))
            {
                MessageBox.Show("請輸入數量 (QCurrent)");
                return;
            }
            int nQCurrent = Convert.ToInt32(txt_TFX_QCurrent.Text.Trim());

            int nQMatch = 0;
            if (IsNumber(txt_TFX_QMatch.Text.Trim()))
            {
                nQMatch = Convert.ToInt32(txt_TFX_QMatch.Text.Trim());
            }

            int nPreOrder = TFX_CheckPreOrder.Checked ? 1 : 0;
            int nNewPT = 1;				// 0:限價, 1:漲停, 2:跌停, 3:平盤, 4:市價, 5:長效單
            string strNewPrice = "0";

            int nNewCond = 0;			// 0:ROD, 1:FOK, 2:IOC

            if (cb_STK_Cond.Text == OrderCondition.ROD)
                nNewCond = 0;
            else if (cb_STK_Cond.Text == OrderCondition.FOK)
                nNewCond = 1;
            else if (cb_STK_Cond.Text == OrderCondition.IOC)
                nNewCond = 2;

            string strCommID1 = txt_TFX_TradeID.Text.Trim();
            string strCommID2 = "";

            int nBS1 = Get_TFX_BS();
            int nBS2 = 0;
            int nCond = 0;
            int nOffSet = 0;

            string strResult = m_tradeApp.FutOpt_ModifyOrder(szAccountID, nType, m_nTradeDate, szOID,
                                szOrderNo, nQty, nTT, nQCurrent, nQMatch, nPreOrder, nNewPT, strNewPrice, nNewCond,
                                strCommID1, strCommID2, jasondata);

            OutToMessage("期貨改單：" + strResult);
        }

        /// <summary>
        /// 刪單
        /// </summary>
        private void btn_TFX_CancelOrder_Click(object sender, EventArgs e)
        {
            m_nTradeDate = Check_TFX_TradeDate();
            if (m_nTradeDate < 0)
            {
                MessageBox.Show("交易日期格式錯誤", "錯誤");
                return;
            }

            //string szAccountID = "15000-3314559";
            string szAccountID = GetAccountBySecuritiesType(Securities.TFX);
            string szOID = txt_TFX_OID.Text.Trim();
            string szOrderNo = txt_TFX_OrderNo.Text.Trim();
            if (!IsNumber(txt_TFX_Qty.Text.Trim()))
            {
                MessageBox.Show("請輸入數量");
                return;
            }
            int nQty = Convert.ToInt32(txt_TFX_Qty.Text.Trim());

            int nTT = Get_TFX_Type();    // 期權商品類型 (0:期貨, 1:選擇權)

            if (!IsNumber(txt_TFX_QCurrent.Text.Trim()))
            {
                MessageBox.Show("請輸入數量 (QCurrent)");
                return;
            }
            int nQCurrent = Convert.ToInt32(txt_TFX_QCurrent.Text.Trim());

            int nQMatch = 0;
            if (IsNumber(txt_TFX_QMatch.Text.Trim()))
            {
                nQMatch = Convert.ToInt32(txt_TFX_QMatch.Text.Trim());
            }
            int nPreOrder = TFX_CheckPreOrder.Checked ? 1 : 0;

            string strCommID1 = txt_TFX_TradeID.Text.Trim();
            string strCommID2 = "";

            //int nBS1 = Get_TFX_BS();

            string strResult = m_tradeApp.FutOpt_CancelOrder(szAccountID, m_nTradeDate, szOID,
                                szOrderNo, nQty, nTT, nQCurrent, nQMatch, nPreOrder,
                                strCommID1, strCommID2, jasondata);

            OutToMessage("期貨刪單：" + strResult);
        }

        #endregion

        #region 查詢

        /// <summary>
        /// 委託查詢
        /// </summary>
        private void btn_TFX_QryOrder_Click(object sender, EventArgs e)
        {
            //string szAccountID = "15000-3314559";
            string szAccountID = GetAccountBySecuritiesType(Securities.TFX);
            int nTT = Get_TFX_Type();    // 期權商品類型 (0:期貨, 1:選擇權)
            int nForceQuery = TFX_CheckForceQuery.Checked ? 1 : 0;

            string strResult = m_tradeApp.FutOpt_QueryOrder(szAccountID, nTT, nForceQuery);

            OutToMessage("期貨委託查詢：" + strResult);
        }

        /// <summary>
        /// 成交查詢
        /// </summary>
        private void btn_TFX_QryMatch_Click(object sender, EventArgs e)
        {
            //string szAccountID = "15000-3314559";
            string szAccountID = GetAccountBySecuritiesType(Securities.TFX);
            int nForceQuery = TFX_CheckForceQuery.Checked ? 1 : 0;

            string strResult = m_tradeApp.FutOpt_QueryMatch(szAccountID, nForceQuery);

            OutToMessage("期貨成交查詢：" + strResult);
        }

        /// <summary>
        /// 未平倉查詢
        /// </summary>
        private void btn_TFX_QryOI_Click(object sender, EventArgs e)
        {
            m_nTradeDate = Check_TFX_TradeDate();
            if (m_nTradeDate < 0)
            {
                MessageBox.Show("交易日期格式錯誤", "錯誤");
                return;
            }

            //string szAccountID = "15000-3314559";
            string szAccountID = GetAccountBySecuritiesType(Securities.TFX);
            int nForceQuery = TFX_CheckForceQuery.Checked ? 1 : 0;

            string strResult = m_tradeApp.FutOpt_QueryOI(szAccountID, m_nTradeDate, nForceQuery);

            OutToMessage("期貨未平倉查詢：" + strResult);
        }

        /// <summary>
        /// 權益查詢
        /// </summary>
        private void btn_TFX_Equity_Click(object sender, EventArgs e)
        {
            string szAccountID = GetAccountBySecuritiesType(Securities.TFX);
            int nForceQuery = TFX_CheckForceQuery.Checked ? 1 : 0;

            string strResult = m_tradeApp.FutOpt_QueryEquity(szAccountID, "");

            OutToMessage("期貨權益查詢：" + strResult);
        }

        #endregion

        #region 其他

        /// <summary>
        /// 檢核交易日期
        /// </summary>
        private int Check_TFX_TradeDate()
        {
            int returnValue = -1;

            txt_TFX_TradeDate.Text = txt_TFX_TradeDate.Text.Trim();
            if (txt_TFX_TradeDate.Text.Length == 8)
            {
                try
                {
                    DateTime.ParseExact(txt_TFX_TradeDate.Text, "yyyyMMdd", null);
                    returnValue = Convert.ToInt32(txt_TFX_TradeDate.Text);
                }
                catch (Exception)
                {
                    returnValue = -1;
                }
            }

            return returnValue;
        }

        /// <summary>
        /// 買/賣
        /// </summary>
        private int Get_TFX_BS()
        {
            int nBS = 2;    // 買賣別 (1: 買進/ 2: 賣出)
            if (cb_TFX_BS.SelectedIndex == 0 || cb_TFX_BS.SelectedIndex == 1)
            {
                nBS = cb_TFX_BS.SelectedIndex + 1;
            }
            return nBS;
        }

        /// <summary>
        /// 期貨、選擇權
        /// </summary>
        private int Get_TFX_Type()
        {
            int nTT = 0;    // 期權商品類型 (0:期貨, 1:選擇權)
            if (cb_TFX_Type.SelectedIndex == 0 || cb_TFX_Type.SelectedIndex == 1)
            {
                nTT = cb_TFX_Type.SelectedIndex;
            }
            return nTT;
        }

        #endregion

        #endregion

        #region Util

        public static bool IsNumber(string strNumber)
        {
            System.Text.RegularExpressions.Regex r = new System.Text.RegularExpressions.Regex(@"^\d+(\.)?\d*$");
            return r.IsMatch(strNumber);
        }

        public static string FormatResult(string strInput)
        { 
            return strInput.Replace("><", ">\n<");
        }

        #endregion

        #region 其他功能測試
        private void btn_UnitTest_Click(object sender, EventArgs e)
        {
            bool isAllOK = true;
            StringBuilder msg = new StringBuilder();

            msg.AppendLine("------------------------------------------------------ Unit Test -------------------------------------------------------------");

            if (m_tradeApp != null)
            {
                if (m_tradeApp.GetAccountList().Length > 0)
                    msg.AppendFormat("{0}：{1}\r\n", "GetAccountList", "正常，" + m_tradeApp.GetAccountList());
                else
                    msg.AppendFormat("{0}：{1}\r\n", "GetAccountList", "異常");

                try
                {
                    m_tradeApp.SetLogPath(".");
                    msg.AppendFormat("{0}：{1}\r\n", "SetLogPath", "正常");
                }
                catch (Exception ex)
                {
                    isAllOK = false;
                    msg.AppendFormat("{0}：{1}\r\n", "SetLogPath", "錯誤，" + ex.Message);
                }

                try
                {
                    m_tradeApp.SetExecOrderType(0);
                    msg.AppendFormat("{0}：{1}\r\n", "SetExecOrderType", "正常");
                }
                catch (Exception ex)
                {
                    isAllOK = false;
                    msg.AppendFormat("{0}：{1}\r\n", "SetExecOrderType", "錯誤，" + ex.Message);
                }

                try
                {
                    m_tradeApp.SetEchoType(1, 0);
                    msg.AppendFormat("{0}：{1}\r\n", "SetEchoType", "正常");
                }
                catch (Exception ex)
                {
                    isAllOK = false;
                    msg.AppendFormat("{0}：{1}\r\n", "SetEchoType", "錯誤，" + ex.Message);
                }

                try
                {
                    m_tradeApp.SetLotSizeData("0000=100");
                    msg.AppendFormat("{0}：{1}\r\n", "SetLotSizeData", "正常");
                }
                catch (Exception ex)
                {
                    isAllOK = false;
                    msg.AppendFormat("{0}：{1}\r\n", "SetLotSizeData", "錯誤，" + ex.Message);
                }
            }
            else
            {
                isAllOK = false;
                msg.AppendLine("尚未初始化 m_tradeApp 物件");
            }

            msg.AppendLine("-------------------------------------------------------------------------------------------------------------------------------");

            txt_Result.AppendText(msg.ToString());

            if (isAllOK)
                MessageBox.Show("測試 - 正常");
            else
                MessageBox.Show("測試 - 含有錯誤項目");
        }
        #endregion

        #region 清除

        private void btn_ClearResult_Click(object sender, EventArgs e)
        {
            txt_Result.Clear();
        }

        private void btn_ClearMSN_Click(object sender, EventArgs e)
        {
            txt_MSN.Clear();
        }

        private void btn_ClearMessage_Click(object sender, EventArgs e)
        {
            txt_Message.Clear();
        }

        #endregion

        private void OutToMessage(string message)
        {
            txt_Message.AppendText(message + "\r\n");
        }

        private void btnMixQuery_Click(object sender, EventArgs e)
        {
            string szAccountID = GetAccountBySecuritiesType("ALL");
            if (txt_API.Text.Length <= 0)
            {
                MessageBox.Show("輸入 API");
                return;
            }
            string strResult = m_tradeApp.MIX_QueryData(szAccountID, txt_API.Text, txt_Param.Text);
            if (strResult != null && strResult.Length > 0)
                MessageBox.Show(strResult);
        }
    }
}
